package org.example;

import com.rabbitmq.client.AMQP;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeoutException;

public class AMQPProducer {
    public static void main(String[] args) throws IOException, TimeoutException {
        // 下列变量按照"接入配置"的说明进行配置
        final String host = "rbtmq-xxxxxxx.rabbitmq.ivolces.com";
        final int port = 5672;
        final String username = "demo-user";
        final String password = "demo-user-password";

        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost(host);
        factory.setPort(port);
        factory.setUsername(username);
        factory.setPassword(password);

        // 配置连接相关的参数，请根据业务特点与网络状况配置
        factory.setAutomaticRecoveryEnabled(true);
        factory.setNetworkRecoveryInterval(5000);
        factory.setConnectionTimeout(30 * 1000);
        factory.setHandshakeTimeout(30 * 1000);
        factory.setShutdownTimeout(0);

        // 下列是消息发布的AMQP-0-9-1协议对象配置
        final String vhost = "/";
        final String exchange = "demo-exchange";
        final String exchangeType = "direct";
        final String queue = "demo-queue";
        final String bindingKey = "foo";

        // 设置Vhost名称，请确保已经在RabbitMQ集群Web控制台创建
        factory.setVirtualHost(vhost);

        // 创建Connection和Channel
        Connection connection = factory.newConnection();
        Channel channel = connection.createChannel();

        // 创建direct类型的exchange，如果已经在RabbitMQ集群Web控制台手动创建，则可跳过此步骤
        channel.exchangeDeclare(exchange, exchangeType, true, false, false, null);
        // 创建Quorum队列并绑定到exchange，如果已经在RabbitMQ集群Web控制台手动创建，则可跳过此步骤
        HashMap<String, Object> arg = new HashMap<>();
        arg.put("x-queue-type", "quorum");
        channel.queueDeclare(queue, true, false, false, arg);
        channel.queueBind(queue, exchange, bindingKey);

        // 开始消息生产
        for (int i = 0; i < 100; i++  ) {
            // BindingKey根据业务需求填入相应的BindingKey。
            AMQP.BasicProperties props = new AMQP.BasicProperties.Builder().messageId(UUID.randomUUID().toString()).build();
            channel.basicPublish(exchange, bindingKey, true, props,
                    ("demo message "  + i).getBytes(StandardCharsets.UTF_8));
        }

        channel.close();
        connection.close();
    }
}